(function ($) {
  try {
    new THREE.WebGLRenderer();
  } catch ( e ) {
    $('.panorama-wrap').html('申し訳ございません。<br>お客様のブラウザはパノラマ画像の表示に対応しておりません。<br>別の最新のブラウザにて再度お確かめください。');
    return false;
  }

  // パノラマ画像
  var PanoramaData = function(imgNo) {
    var _ = this;

    _.imgNo = imgNo;
    _.element;                    // パノラマ画像のキャンバス
    _.scene;
    _.camera;
    _.geometry;
    _.material;
    _.renderer;
    _.sphere;
    _.controls;
    _.controlsDeviceOrientation;  // ジャイロセンサー
  };

  // キャンバスのサイズを取得
  PanoramaData.prototype.getWidthHeight = function(image) {
    var _ = this;

    var target = image.querySelector('.panorama-stage');
    return [target.clientWidth, target.clientHeight];
  };

  // 視点を初期化
  PanoramaData.prototype.resetCameraPosition = function() {
    var _ = this;

    _.camera.position.set(3, 0.15, 0.15);
    _.controls.target.set(0.15, 0, 0);
  };

  PanoramaData.prototype.init = function(image, initHandler) {
    var _ = this;

    // キャンバスのサイズを取得
    var widthHeight = _.getWidthHeight(image);
    var width = widthHeight[0];
    var height = widthHeight[1];

    // シーンの作成
    _.scene = new THREE.Scene();

    // カメラの作成
    _.camera = new THREE.PerspectiveCamera(60, width / height, 1, 1000);
    _.camera.position.set(3, 0.15, 0.15);
    _.scene.add(_.camera);

    // 球体の形状を作成
    _.geometry = new THREE.SphereGeometry(5, 60, 40);
    _.geometry.scale(-1, 1, 1);

    // 画像データ取得
    var loadFile = image.querySelector('.active img').getAttribute('data-src');

    var loader = new THREE.TextureLoader();
    loader.load(loadFile, function(texture) {
      texture.minFilter = THREE.LinearFilter;
      texture.magFilter = THREE.LinearFilter;

      //　マテリアルの作成
      _.material = new THREE.MeshBasicMaterial({
        map: texture
      });

      // 球体(形状)にマテリアル(質感)を貼り付けて物体を作成
      _.sphere = new THREE.Mesh(_.geometry, _.material);

      //　シーンに追加
      _.scene.add(_.sphere);

      // レンダラーの作成
      _.renderer = new THREE.WebGLRenderer();
      // レンダラーをwindowサイズに合わせる
      _.renderer.setSize(width, height);
      _.renderer.setClearColor(0xDCDCDC, 1.0);
      _.element = _.renderer.domElement;

      // 視点操作
      _.setOrbitControls(image);

      // ジャイロ
      _.setDeviceOrientationControls();

      // 呼び出し元で指定された初期化完了処理を行う
      initHandler();
    });
  };

  // スマートフォン向けジャイロセンサー機能
  PanoramaData.prototype.setDeviceOrientationControls = function() {
    var _ = this;

    _.controlsDeviceOrientation = new THREE.DeviceOrientationControls(_.camera);
  };

  // 移動・回転の制御
  PanoramaData.prototype.setOrbitControls = function(image) {
    var _ = this;

    _.controls = new THREE.OrbitControls(_.camera, _.element);
    _.controls.target.set(0.15, 0, 0);

    // 視点操作のイージングをONにする
    _.controls.enableDamping = true;
    // 視点操作のイージングの値
    _.controls.dampingFactor = 0.2;
    // ローテーションの速さ
    _.controls.rotateSpeed = wpvGl.rotateSpeed;
    // ズーム速度
    _.controls.zoomSpeed = wpvGl.zoomSpeed;
    // ズームの範囲
    _.controls.minDistance = wpvGl.minDistance;
    _.controls.maxDistance = wpvGl.maxDistance;
    // パン操作禁止
    _.controls.enablePan = false;
    // オートローテーション
    if (wpvGl.autolotation === true) {
      _.controls.autoRotate = true;
      // 停止ボタンに切り替える
      image.querySelector('.panorama-control').classList.remove('play');
      image.querySelector('.panorama-control').classList.add('pause');
    } else if (wpvGl.autolotation === false) {
      _.controls.autoRotate = false;
      // プレイボタンに切り替える
      image.querySelector('.panorama-control').classList.remove('pause');
      image.querySelector('.panorama-control').classList.add('play');
    }
    _.controls.autoRotateSpeed = wpvGl.autoRotateSpeed;
  };

  PanoramaData.prototype.render = function() {
    var _ = this;
    _.renderer.render(_.scene, _.camera);
  };

  PanoramaData.prototype.handleResize = function(image) {
    var _ = this;

    // キャンバスのサイズを取得
    var widthHeight = _.getWidthHeight(image);
    var width = widthHeight[0];
    var height = widthHeight[1];

    _.renderer.setSize(width, height);
    _.render();
    _.camera.aspect = width / height;
    _.camera.updateProjectionMatrix();
  }

  // パノラマ画像データの削除処理
  PanoramaData.prototype.delete = function() {
    var _ = this;

    if (_.scene) {
      _.scene.remove(_.sphere);
      _.scene.remove(_.camera);
      _.scene = null;
    }
    _.camera = null;
    _.sphere = null;

    if (_.controls) {
      _.controls.dispose();
      _.controls = null;
    }
    if (_.controlsDeviceOrientation) {
      _.controlsDeviceOrientation.dispose();
      _.controlsDeviceOrientation = null;
    }
    if (_.geometry) {
      _.geometry.dispose();
      _.geometry = null;
    }

    if (_.material) {
      _.material.map.dispose();
      _.material.map.image = null;
      _.material.map = null;
      _.material.dispose();
      _.material = null;
    }

    if (_.renderer) {
      _.renderer.dispose();
      _.renderer.forceContextLoss();
      _.renderer.context = null;
      _.renderer.domElement = null;
      _.renderer = null;
    }
  };

  // パノラマ画像データのキャッシュ
  var DataCache = function() {
    var _ = this;

    _.maxNum = 5;
    _.data = {};
    _.dataKeys = [];
  };

  // 保持しているパノラマ画像データを返す
  DataCache.prototype.getData = function(key) {
    var _ = this;

    var result = _.data[key];
    if (result) {
      // 該当データが既にある場合は、キャッシュから削除される順番を最後に回す
      _.dataKeys.splice(_.dataKeys.indexOf(key), 1);
      _.dataKeys.push(key);
    }

    return result;
  };

  // パノラマ画像データをキャッシュに追加する
  DataCache.prototype.push = function(key, data) {
    var _ = this;

    if (_.maxNum <= _.dataKeys.length) {
      // 保存するデータの最大数を超える場合は、dataKeysの先頭から一つ削除する
      var deleting = _.dataKeys.shift();
      _.data[deleting].delete();
      delete _.data[deleting];
    }
    _.data[key] = data;
    _.dataKeys.push(key);
  };

  var PanoramaView = function(image) {
    var _ = this;

    _.image = image;
    _.windowHeight = window.innerHeight;
    _.rect = _.image.getBoundingClientRect();

    _.frameId;
    _.initFlag = false;           // セッティング完了フラグ
    _.renderStart = null;         // レンダリングとその時間の制御
    _.initTimer = null;
    _.initStartFlag = false;
    _.sliderSet = null;           // スライダー
    _.fullscreen = null;          // フルスクリーンモード
    _.currentImageNum = 0         // 選択中の画像番号
    _.thumbnailSwitchFlag = false // 選択中の画像番号
    _.panoramaData = null;
    _.panoramaDataCache = null;
  };

  PanoramaView.prototype.initView = function() {
    var _ = this;

    // パノラマ画像データ作成
    _.newPanoramaData(function() {
      var _ = this;

      // canvasを追加してレンダリング
      _.image.querySelector('.panorama-stage').innerHTML = '';
      _.image.querySelector('.panorama-stage').appendChild(_.panoramaData.element);
      _.panoramaData.render();

      // パノラマ画像データ作成後の処理
      _.afterInit();
    });
  };

  // パノラマ画像データ作成
  PanoramaView.prototype.newPanoramaData = function(callback) {
    var _ = this;

    var imgNo = _.image.querySelector('.active img').getAttribute('data-no');
    _.panoramaData = new PanoramaData(imgNo);
    _.panoramaData.init(_.image, callback.bind(_));

    return _.panoramaData;
  };

  // パノラマ画像データ作成後の処理
  PanoramaView.prototype.afterInit = function() {
    var _ = this;

    // ローディングアイコンの非表示
    $(_.image).find('.panorama-loading').fadeOut(500);

    // セッティング完了
    _.initFlag = true;
  };

  // アニメーションの実行
  PanoramaView.prototype.renderLoop = function(timestamp) {
    var _ = this;

    // 実行時間をセット
    if (!_.renderStart) {
      _.renderStart = timestamp;
    }
    // 実行時間の計測
    var progress = timestamp - _.renderStart;

    if (progress < (5 * 1000 * 60)) {
      // 実行時間が5分を過ぎていない場合はアニメーションを継続
      _.frameId = requestAnimationFrame(_.renderLoop.bind(_));

      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }
    } else {
      // 実行時間が5分を過ぎた場合はアニメーションを終了
      cancelAnimationFrame(_.frameId);

      _.panoramaData.controls.autoRotate = false;
      _.image.querySelector('.panorama-control').classList.remove('pause');
      _.image.querySelector('.panorama-control').classList.add('play');
    }

    if ($(_.image).find('.panorama-gyro').hasClass('on')) {
      // フルスクリーン時にジャイロセンサーのアイコンをクリックした場合は、ジャイロモードに切り替える
      _.panoramaData.controlsDeviceOrientation.update();
    } else {
      // ジャイロモードでない場合は、通常の画像移動・回転を行う
      _.panoramaData.controls.update();
    }
    _.panoramaData.render();
  };

  // リサイズ処理
  PanoramaView.prototype.handleResize = function() {
    var _ = this;

    // セッティングが完了していない場合は何もしない
    if (!_.initFlag) {
      return;
    }

    _.panoramaData.handleResize(_.image);
    _.handleResizeSlider();
  };

  PanoramaView.prototype.renderUpdate = function() {
    var _ = this;

    var newImgNo = _.image.querySelector('.active img').getAttribute('data-no');

    // パノラマ画像データのキャッシュが無い場合作成する
    if (!_.panoramaDataCache) {
      _.panoramaDataCache = new DataCache();
      // 切り替え前のパノラマ画像をキャッシュへ格納
      _.panoramaDataCache.push(_.panoramaData.imgNo, _.panoramaData);
    }

    // 一旦アニメーションを止める
    cancelAnimationFrame(_.frameId);

    // 切り替え先のパノラマ画像データをキャッシュから取得
    var newPanoramaData = _.panoramaDataCache.getData(newImgNo);
    if (newPanoramaData) {
      // 切り替え先のパノラマ画像データへ切り替え
      _.panoramaData = newPanoramaData;
      // キャッシュ保存時とサイズが変わっている可能性があるのでリサイズ処理を実行
      _.panoramaData.handleResize(_.image);
      // canvasが非表示になっているので表示させる
      $(_.panoramaData.element).show();
      // 移動・回転を有効にする
      _.panoramaData.setOrbitControls(_.image);
      _.afterUpdate();
    } else {
      // パノラマ画像データがキャッシュに無い場合は作成する（-> _.panoramaDataに格納される）
      _.newPanoramaData(_.afterUpdate);
      // 作成したパノラマ画像データをキャッシュに追加しておく
      _.panoramaDataCache.push(newImgNo, _.panoramaData);
    }
  };

  // パノラマ画像切り替え後の処理
  PanoramaView.prototype.afterUpdate = function() {
    var _ = this;

    // 視点操作初期化
    _.panoramaData.resetCameraPosition();

    // アニメーションを開始
    _.renderStart = null;
    cancelAnimationFrame(_.frameId);
    _.frameId = requestAnimationFrame(_.renderLoop.bind(_));

    // パノラマ画像データ作成後の処理を実行
    _.afterInit();

    // canvasを先頭に追加する
    _.image.querySelector('.panorama-stage').insertBefore(_.panoramaData.element, _.image.querySelector('.panorama-stage canvas'));

    // 元々表示していたcanvasを徐々にフェードアウトし削除
    var garbageCanvas = $(_.image).find('.panorama-stage canvas:last');
    garbageCanvas.fadeOut(800).queue(function() {
      garbageCanvas.remove();

      // サムネイル切り替え完了
      _.thumbnailSwitchFlag = false;
    });
  };

  // リサイズイベントを検知してリサイズ処理を実行
  PanoramaView.prototype.addEventListener = function() {
    var _ = this;

    var resizeQueue = null;
    // ブラウザサイズを変更した場合にパノラマ画像要素のリサイズを行う
    window.addEventListener('resize', function() {
      clearTimeout(resizeQueue);
      resizeQueue = setTimeout(_.handleResize.bind(_), 100);
    });

    // パノラマ画像のレンダリングを開始する
    window.addEventListener('load', _.initRender.bind(_), false);
    window.addEventListener('scroll', _.initRender.bind(_), false);

    // ウィンドウからフォーカスが外れた場合にアニメーションをストップする
    window.addEventListener('blur', function() {
      if (typeof _.frameId !== 'undefined') {
        cancelAnimationFrame(_.frameId);
      }
    });
    // ウィンドウにフォーカスが当たった場合にアニメーションを再開する
    window.addEventListener('focus', function() {
      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }

      _.renderStart = null;
      cancelAnimationFrame(_.frameId);
      _.frameId = requestAnimationFrame(_.renderLoop.bind(_));
    });
  };

  // パノラマビューアーにフォーカスされた場合に、描画時間をリセットして延長する
  PanoramaView.prototype.initClickExtension = function() {
    var _ = this;

    $(_.image).on('mousedown touchstart', function() {
      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }

      _.renderStart = null;
      cancelAnimationFrame(_.frameId);
      _.frameId = requestAnimationFrame(_.renderLoop.bind(_));
    });
  };

  // サムネイル付きパノラマ画像セットのサムネイルクリック時の処理
  PanoramaView.prototype.initThumbnailClick = function() {
    var _ = this;

    $(_.image).find('ul li').on('click', function() {
      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }

      // サムネイルを切り替え中の場合は何もしない
      if (_.thumbnailSwitchFlag) {
        return;
      }

      // 選択中のパノラマ画像を選択した場合は何もしない
      var imgNo = $('img', this).attr('data-no');
      if (_.panoramaData.imgNo == imgNo) {
        return;
      }

      // クリックしたサムネイルをパノラマ表示する
      $(this).addClass('active');
      $(this).siblings().removeClass('active');

      // 表示切替開始 //
      _.initFlag = false;
      _.thumbnailSwitchFlag = true;

      // ローディングアイコンを表示
      $(_.image).find('.panorama-loading').show();

      // 切り替え中、移動・回転は無効にする
      _.panoramaData.controls.dispose();
      _.panoramaData.controls = null;

      _.renderUpdate();
    });
  };

  // 制御ボタンの制御
  PanoramaView.prototype.initControlClick = function() {
    var _ = this;

    $(_.image).find('.panorama-control').on('click', function() {
      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }

      // 再生ボタン
      if (this.classList.contains('play')) {
        _.panoramaData.controls.autoRotate = true;
        $(this).removeClass('play').addClass('pause');
      } else {
        _.panoramaData.controls.autoRotate = false;
        $(this).removeClass('pause').addClass('play');
      }
    });

    // フルスクリーンボタン
    $(_.image).find('.panorama-fullscreen').on('click', function() {
      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }

      var targetScreen = _.image.querySelector('.panorama-stage-wrap');
      var targetIcon = this;
      _.fullscreen.toggleFullscreen(targetScreen, targetIcon);
    });

    // ジャイロモードボタン
    $(_.image).find('.panorama-gyro').on('click', function() {
      if (!_.initFlag) {
        return;
      }

      _.toggleGyro();
    })
  };

  // ジャイロモードの切り替え
  PanoramaView.prototype.toggleGyro = function() {
    var _ = this;

    if (!(window.DeviceOrientationEvent && 'ontouchstart' in window)) {
      return false;
    }

    var gyroIcon = $(_.image).find('.panorama-gyro');
    var controlIcon = $(_.image).find('.panorama-control');
    if ($(gyroIcon).hasClass('on')) {
      $(gyroIcon).removeClass('on');
      $(gyroIcon).attr('title', 'ジャイロ');
      // 再生ボタン表示
      controlIcon.show();

      // 視点操作
      _.panoramaData.setOrbitControls(_.image);

      // 視点初期化
      _.panoramaData.resetCameraPosition();
    } else {
      $(gyroIcon).addClass('on');
      $(gyroIcon).attr('title', '解除');
      // 再生ボタン非表示
      controlIcon.hide();

      // 通常のモードを解除
      _.panoramaData.controls.dispose();
    }
  }

  // フルスクリーンモードの処理
  var Fullscreen = function(image) {
    var _ = this;
    _.image = image;
  };

  Fullscreen.prototype.init = function() {
    var _ = this;

    if (_.enableFullscreen()) {
      _.addEventListener();
    } else {
      $(_.image).find('.panorama-fullscreen').hide();
    }
  };

  // フルスクリーンに関するイベントリスナーの登録
  Fullscreen.prototype.addEventListener = function() {
    var _ = this;

    document.addEventListener("webkitfullscreenchange", _.changeFullscreen.bind(_), false);
    document.addEventListener("fullscreenchange", _.changeFullscreen.bind(_), false);
    document.addEventListener("mozfullscreenchange", _.changeFullscreen.bind(_), false);
    document.addEventListener("msfullscreenchange", _.changeFullscreen.bind(_), false);
    document.addEventListener("__iosfullscreenchange", _.changeFullscreen.bind(_), false);
  };

  // フルスクリーンが利用可能か判定
  Fullscreen.prototype.enableFullscreen = function() {
    return document.fullscreenEnabled ||
          document.webkitFullscreenEnabled ||
          document.mozFullScreenEnabled ||
          document.msFullscreenEnabled ||
		  document.__iosFullscreenEnabled ||
          false;
  };

  // フルスクリーン状態にあるか判定
  Fullscreen.prototype.isFullscreen = function() {
    return document.fullscreenElement ||
          document.webkitFullscreenElement ||
          document.mozFullScreenElement ||
          document.msFullscreenElement ||
          document.__iosFullscreenElement ||
          false;
  };

  // フルスクリーンに切り替え
  Fullscreen.prototype.requestFullscreen = function(target) {
    if (target.requestFullscreen) {
      target.requestFullscreen();
    } else if (target.webkitRequestFullscreen) {
      target.webkitRequestFullscreen();
    } else if (target.mozRequestFullScreen) {
      target.mozRequestFullScreen();
    } else if (target.msRequestFullscreen) {
      target.msRequestFullscreen();
    } else if (target.__iosRequestFullscreen) {
      target.__iosRequestFullscreen();
    }
  };

  // フルスクリーンの解除
  Fullscreen.prototype.cancelFullscreen = function() {
    if (document.webkitCancelFullScreen) {
      document.webkitCancelFullScreen();
    } else if (document.mozCancelFullScreen) {
      document.mozCancelFullScreen();
    } else if (document.msExitFullscreen) {
      document.msExitFullscreen();
    } else if (document.cancelFullScreen) {
      document.cancelFullScreen();
    } else if (document.exitFullscreen) {
      document.exitFullscreen();
    } else if (document.__iosCancelFullScreen) {
      document.__iosCancelFullScreen();
    }
  };

  // フルスクリーンが切り替わった場合に行う処理
  Fullscreen.prototype.changeFullscreen = function() {
    var _ = this;

    targetIcon = $(_.image).find('.panorama-fullscreen');
    gyroIcon = $(_.image).find('.panorama-gyro');
    if (_.isFullscreen()) {
      $(targetIcon).addClass('cancel');
      $(targetIcon).attr('title', '元に戻す');

      // ジャイロ機能が有効で、タッチデバイスの場合にジャイロボタンを表示
      if (window.DeviceOrientationEvent && 'ontouchstart' in window) {
        $(gyroIcon).show();
      }
    } else {
      $(targetIcon).removeClass('cancel');
      $(targetIcon).attr('title', '最大化');

      // ジャイロボタンの非表示
      if (window.DeviceOrientationEvent && 'ontouchstart' in window) {
        $(gyroIcon).hide();
        if ($(gyroIcon).hasClass('on')) {
          // ジャイロモードのままフルスクリーンが解除された場合は、ジャイロモードも解除する
          gyroIcon.click();
        }
      }
    }
  };

  // フルスクリーンの切り替えを行う
  Fullscreen.prototype.toggleFullscreen = function(targetScreen) {
    var _ = this;

    if (!_.enableFullscreen()) {
      return false;
    }

    if (_.isFullscreen()) {
      _.cancelFullscreen();
    } else {
      _.requestFullscreen(targetScreen);
    }
  };

  // 画像データを取得する
  PanoramaView.prototype.loadImage = function() {
    var _ = this;

    var imageList = _.image.getElementsByClassName('wpv-lazyload');
    var images = Array.prototype.slice.call(imageList, 0);
    images.forEach(function(image) {
      var imagePath = image.getAttribute('data-thumbsrc');
      image.setAttribute('src', imagePath);
    });
  };

  // パノラマ画像のレンダリングをスタート
  PanoramaView.prototype.initRender = function() {
    var _ = this;

    if (_.initStartFlag) {
      return;
    }

    _.rect = this.image.getBoundingClientRect();
    clearTimeout(_.initTimer);
    _.initTimer = setTimeout(function() {
      if (_.windowHeight > _.rect.top) {
        _.initStartFlag = true;
        _.loadImage();
        _.initView();
        _.frameId = _.renderLoop(0);
        _.sliderSet = new Slider(_.image);
        if (_.sliderSet.isOverflow($('.slider'))) {
          _.sliderSet.slideBtnToggle();
        }
      }
    }, 100);
  };

  // ブラウザリサイズ時のサムネイルのスライドボタンの表示・非表示
  PanoramaView.prototype.handleResizeSlider = function() {
    var _ = this;

    if (!_.sliderSet) {
      return;
    }

    var isOver = _.sliderSet.isOverflow($('.slider'));
    var change = (isOver == _.sliderSet.overflowSwitch);
    if (change && isOver) {
      if (_.sliderSet.slideCurrent !== 0) {
        _.sliderSet.slideCurrent = 0;
        _.sliderSet.sliding();
      }
      _.sliderSet.slideBtnToggle();
      _.sliderSet.overflowSwitch = false;
    } else if (change && !isOver) {
      if (_.sliderSet.slideCurrent !== 0) {
        _.sliderSet.slideCurrent = 0;
        _.sliderSet.sliding();
      }
      $(_.image).find('.slider .slider-prev').hide();
      $(_.image).find('.slider .slider-next').hide();
      _.sliderSet.overflowSwitch = true;
    }
    if (_.sliderSet.fitWidth($('.slider'))) {
      $(_.image).find('.slider .slider-next').hide();
    } else {
      $(_.image).find('slider .slider-next').show();
    }
  };

  // サムネイルクリック時にスライドさせる
  PanoramaView.prototype.thumbnailSliding = function() {
    var _ = this;

    $(_.image).find('.slider .slider-prev').on('click', function() {
      if (!_.sliderSet) {
        return;
      }

      _.sliderSet.slideCurrent--;
      _.sliderSet.sliding();
    });

    $(_.image).find('.slider .slider-next').on('click', function() {
      if (!_.sliderSet) {
        return;
      }

      _.sliderSet.slideCurrent++;
      _.sliderSet.sliding();
    });
  };

  // パノラマ画像の初期設定
  PanoramaView.prototype.init = function() {
    var _ = this;

    _.addEventListener();
    _.initClickExtension();
    _.initThumbnailClick();
    _.initControlClick();
    _.thumbnailSliding();

    // フルスクリーンモードの設定
    _.fullscreen = new Fullscreen(_.image);
    _.fullscreen.init();
  };

  // パノラマ画像のサムネイルの設定
  var Slider = function(image) {
    var _ = this;

    _.image = image;
    _.overflowSwitch = true;
    _.padding = 10;
    _.slideWidth = $(_.image).find('.slide').outerWidth();
    _.slideNum = $(_.image).find('.slide').length;
    _.slideCurrent = 0;
    _.overflowWidth = (_.slideWidth + _.padding) * _.slideNum;
  };

  // パノラマ画像のラッパー要素の幅とサムネイルの合計の幅を比較
  Slider.prototype.isOverflow = function(target) {
    var _ = this;

    return (target.outerWidth() < _.overflowWidth);
  };

  // パノラマ画像のラッパー要素の幅と表示中のサムネイルの幅を比較
  Slider.prototype.fitWidth = function(target) {
    var _ = this;

    var tmpWidth = (_.slideWidth + _.padding) * (_.slideNum - _.slideCurrent);
    return (target.outerWidth() > tmpWidth);
  };

  // パノラマ画像のサムネイルのスライドボタンの表示・非表示
  Slider.prototype.slideBtnToggle = function() {
    var _ = this;

    if (_.slideCurrent === 0) {
      $(_.image).find('.slider .slider-prev').hide();
    }
    if (_.fitWidth($('.slider'))) {
      $(_.image).find('.slider .slider-next').hide();
    } else {
      $(_.image).find('.slider .slider-next').show();
    }
    if (_.slideCurrent > 0) {
      $(_.image).find('.slider .slider-prev').show();
    }
  };

  // パノラマ画像のサムネイルをスライドさせる
  Slider.prototype.sliding = function() {
    var _ = this;

    $(_.image).find('.slideSet').stop().animate({
      left: _.slideCurrent * -(_.slideWidth + _.padding)
    });

    _.slideBtnToggle();
  };

  // パノラマ画像を表示ラッパー要素を取得
  var imageList = document.querySelectorAll('.panorama-wrap');
  // オブジェクト→配列
  var images = Array.prototype.slice.call(imageList, 0);
  // パノラマ画像をそれぞれ表示
  images.forEach(function(image) {
    var panorama = new PanoramaView(image);
    panorama.init();
  });
})(jQuery);
